package com.ruoyi.advertiser.controller;

import com.ruoyi.advertiser.domain.AdHandle;
import com.ruoyi.advertiser.service.IAdHandleService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.List;

/**
 * 把手 Controller
 *
 * @author zhangzhanwei
 * @date 2021-09-15
 */
@RestController
@RequestMapping("/advertising/handle")
public class AdHandleController extends BaseController {

    @Resource
    private IAdHandleService adHandleService;

    /**
     * 查询把手 列表
     */
    @GetMapping("/list")
    public TableDataInfo list(AdHandle adHandle)
    {
        startPage();
        List<AdHandle> list = adHandleService.selectAdHandleList(adHandle);
        return getDataTable(list);
    }

    /**
     * 获取把手 详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(adHandleService.selectAdHandleById(id));
    }

    /**
     * 新增把手
     */
    @Log(title = "把手 ", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AdHandle adHandle)
    {
        if (UserConstants.NOT_UNIQUE.equals(adHandleService.checkNumUnique(adHandle))) {
            return AjaxResult.error("新增把手'" + adHandle.getNum() + "'失败，编号已存在");
        }
        return toAjax(adHandleService.insertAdHandle(adHandle));
    }

    /**
     * 修改把手
     */
    @Log(title = "把手 ", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AdHandle adHandle)
    {
        if (UserConstants.NOT_UNIQUE.equals(adHandleService.checkNumUnique(adHandle))) {
            return AjaxResult.error("修改把手'" + adHandle.getNum() + "'失败，编号已存在");
        }
        return toAjax(adHandleService.updateAdHandle(adHandle));
    }

    /**
     * 删除把手
     */
    @Log(title = "把手 ", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(adHandleService.deleteAdHandleByIds(ids));
    }

}
