package com.ruoyi.advertiser.controller;

import com.ruoyi.advertiser.domain.Material;
import com.ruoyi.advertiser.service.IMaterialService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * 广告素材 Controller
 *
 * @author zhangzhanwei
 * @date 2021-09-13
 */
@RestController
@RequestMapping("/advertising/material")
public class MaterialController extends BaseController {

    @Resource
    private IMaterialService materialService;

    /**
     * 查询广告素材 列表
     */
    @GetMapping("/list")
    public TableDataInfo list(Material material) {
        startPage();
        List<Material> list = materialService.selectMaterialList(material);
        return getDataTable(list);
    }

    /**
     * 新增广告素材
     */
    @Log(title = "广告素材 ", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody Material material) {
        return toAjax(materialService.insertMaterial(material));
    }

    /**
     * 获取广告素材 详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(materialService.selectMaterialById(id));
    }

    /**
     * 修改广告素材
     */
    @Log(title = "广告素材 ", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody Material material) {
        return toAjax(materialService.updateMaterial(material));
    }

    /**
     * 删除广告素材
     */
    @Log(title = "广告素材 ", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(materialService.deleteMaterialByIds(ids));
    }

    /**
     * 图片签名
     */
    @Log(title = "广告素材 ", businessType = BusinessType.UPDATE)
    @PutMapping("/pass")
    public AjaxResult pass(@RequestBody Material Material) {
        return materialService.passMaterial(Material);
    }

}
