package com.ruoyi.advertiser.mapper;

import com.ruoyi.advertiser.domain.CarouselPlan;
import com.ruoyi.advertiser.domain.dto.CarouselPlanDto;
import com.ruoyi.advertiser.domain.req.CarouselPlanParamReq;
import org.apache.ibatis.annotations.Param;
import java.util.Date;
import java.util.List;

/**
 * 广告轮播计划 Mapper接口
 *
 * @author zhangzhanwei
 * @date 2021-09-24
 */
public interface CarouselPlanMapper
{
    /**
     * 查询广告轮播计划
     *
     * @param id 广告轮播计划 主键
     * @return 广告轮播计划
     */
    public CarouselPlan selectAdCarouselPlanById(Long id);

    /**
     * 新增广告轮播计划
     *
     * @param adCarouselPlan 广告轮播计划
     * @return 结果
     */
    public int insertAdCarouselPlan(CarouselPlan adCarouselPlan);

    /**
     * 修改广告轮播计划
     *
     * @param adCarouselPlan 广告轮播计划
     * @return 结果
     */
    public int updateAdCarouselPlan(CarouselPlan adCarouselPlan);

    /**
     * 删除广告轮播计划
     *
     * @param id 广告轮播计划 主键
     * @return 结果
     */
    public int deleteAdCarouselPlanById(Long id);

    /**
     * 批量删除广告轮播计划
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteAdCarouselPlanByIds(Long[] ids);

    /**
     * 查询该周是否有数据
     * @param lineId
     * @param beginOfWeek
     * @param endOfWeek
     * @return
     */
    Integer selectAdCarouselPlanCounts(@Param("lineId") Long lineId,
                                       @Param("beginOfWeek") Date beginOfWeek,
                                       @Param("endOfWeek") Date endOfWeek);

    /**
     * 批量新增广告轮播计划数据
     * @param carouselPlans
     * @return
     */
    Integer insertAdCarouselPlanBatch(List<CarouselPlan> carouselPlans);

    CarouselPlanDto selectAdCarouselPlanByAdCarouselPlanParamVo(CarouselPlanParamReq carouselPlanParamVo);

    /**
     * 根据拉手的imei和开始结束时间查询轮播计划
     *
     * @param imei        拉手imei
     * @param beginOfWeek 周开始时间
     * @param endOfWeek   周结束时间
     * @return
     */
    CarouselPlan selectAdCarouselPlanByImei(@Param("imei") String imei,
                                              @Param("beginOfWeek") Date beginOfWeek,
                                              @Param("") Date endOfWeek);

    /**
     * 说明：删除时间段内轮播计划
     * @param startTime
     * @param endTime
     * @return
     */
    Integer deleteAdCarouselPlanByTime(@Param("startTime") Date startTime,
                                       @Param("endTime") Date endTime);

    int deleteAdCarouselPlanByTimeAndSubwayId(@Param("subwayId") Long subwayId,
                                    @Param("startTime") Date startTime,
                                   @Param("endTime") Date endTime);

    /**
     * 说明：根据列车id和日期查询轮播计划
     * @param subwayId
     * @param day
     * @return
     */
    CarouselPlan selectAdCarouselPlanBySubwayIdAndTime(@Param("subwayId") Long subwayId,
                                                         @Param("day") Date day);
}
