package com.ruoyi.advertiser.service.impl;

import com.ruoyi.advertiser.domain.AdHandle;
import com.ruoyi.advertiser.mapper.AdHandleMapper;
import com.ruoyi.advertiser.service.IAdHandleService;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.service.ISysConfigService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 把手 Service业务层处理
 *
 * @author zhangzhanwei
 * @date 2021-09-15
 */
@Service
public class AdHandleServiceImpl implements IAdHandleService {
    private static final Logger log = LoggerFactory.getLogger(AdHandleServiceImpl.class);

    @Resource
    private AdHandleMapper adHandleMapper;

    /**
     * 查询把手
     *
     * @param id 把手 ID
     * @return 把手
     */
    @Override
    public AdHandle selectAdHandleById(Long id)
    {
        return adHandleMapper.selectAdHandleById(id);
    }

    @Override
    public AdHandle selectAdHandleByImei(String imei) {
        return adHandleMapper.selectAdHandleByImei(imei);
    }

    /**
     * 查询把手 列表
     *
     * @param adHandle 把手
     * @return 把手
     */
    @Override
    public List<AdHandle> selectAdHandleList(AdHandle adHandle)
    {
        List<AdHandle> list = adHandleMapper.selectAdHandleList(adHandle);
        return list;
    }

    /**
     * 新增把手
     *
     * @param adHandle 把手
     * @return 结果
     */
    @Override
    public int insertAdHandle(AdHandle adHandle)
    {
        adHandle.setCreateBy(SecurityUtils.getUsername());
        adHandle.setCreateTime(DateUtils.getNowDate());
        return adHandleMapper.insertAdHandle(adHandle);
    }

    /**
     * 修改把手
     *
     * @param adHandle 把手
     * @return 结果
     */
    @Override
    public int updateAdHandle(AdHandle adHandle)
    {
        adHandle.setUpdateBy(SecurityUtils.getUsername());
        adHandle.setUpdateTime(DateUtils.getNowDate());
        return adHandleMapper.updateAdHandle(adHandle);
    }

    /**
     * 批量删除把手
     *
     * @param ids 需要删除的把手 ID
     * @return 结果
     */
    @Override
    public int deleteAdHandleByIds(Long[] ids)
    {
        return adHandleMapper.deleteAdHandleByIds(ids);
    }

    /**
     * 删除把手 信息
     *
     * @param id 把手 ID
     * @return 结果
     */
    @Override
    public int deleteAdHandleById(Long id)
    {
        return adHandleMapper.deleteAdHandleById(id);
    }

    /**
     * 验证编号是否重复
     */
    @Override
    public String checkNumUnique(AdHandle adHandle) {
        int count = adHandleMapper.checkNumUnique(adHandle);
        return count > 0 ? UserConstants.NOT_UNIQUE : UserConstants.UNIQUE;
    }

    /**
     * 根据心跳修改拉手信息
     */
    @Override
    public void updateHandleByHeartbeat(AdHandle adHandle) {
        adHandleMapper.updateHandleByHeartbeat(adHandle);
    }

    /**
     * 修改seId
     */
    @Override
    public void updateSeId(String imei, String seId) {
        adHandleMapper.updateSeId(imei, seId);
    }

    /**
     * 更新关联关系
     */
    public int updateAdHandleRelation(AdHandle adHandle) {
        return adHandleMapper.updateAdHandleRelation(adHandle);
    }
}
