package com.ruoyi.advertiser.service.impl;

import com.ruoyi.advertiser.domain.AdHeartbeatData;
import com.ruoyi.advertiser.mapper.AdHeartbeatDataMapper;
import com.ruoyi.advertiser.service.IAdHeartbeatDataService;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.service.ISysConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Calendar;
import java.util.List;

/**
 * 拉手心跳数据Service业务层处理
 *
 * @author zhangzhanwei
 * @date 2021-12-16
 */
@Service
public class AdHeartbeatDataServiceImpl implements IAdHeartbeatDataService
{
    @Resource
    private AdHeartbeatDataMapper adHeartbeatDataMapper;

    /**
     * 查询拉手心跳数据
     *
     * @param id 拉手心跳数据主键
     * @return 拉手心跳数据
     */
    @Override
    public AdHeartbeatData selectAdHeartbeatDataById(Long id)
    {
        return adHeartbeatDataMapper.selectAdHeartbeatDataById(id);
    }

    /**
     * 查询拉手心跳数据列表
     *
     * @param adHeartbeatData 拉手心跳数据
     * @return 拉手心跳数据
     */
    @Override
    public List<AdHeartbeatData> selectAdHeartbeatDataList(AdHeartbeatData adHeartbeatData)
    {
        return adHeartbeatDataMapper.selectAdHeartbeatDataList(adHeartbeatData);
    }



    /**
     * 新增拉手心跳数据
     *
     * @param adHeartbeatData 拉手心跳数据
     * @return 结果
     */
    @Override
    public int insertAdHeartbeatData(AdHeartbeatData adHeartbeatData)
    {
        return adHeartbeatDataMapper.insertAdHeartbeatData(adHeartbeatData);
    }

    /**
     * 修改拉手心跳数据
     *
     * @param adHeartbeatData 拉手心跳数据
     * @return 结果
     */
    @Override
    public int updateAdHeartbeatData(AdHeartbeatData adHeartbeatData)
    {
        adHeartbeatData.setUpdateTime(DateUtils.getNowDate());
        return adHeartbeatDataMapper.updateAdHeartbeatData(adHeartbeatData);
    }

    /**
     * 批量删除拉手心跳数据
     *
     * @param ids 需要删除的拉手心跳数据主键
     * @return 结果
     */
    @Override
    public int deleteAdHeartbeatDataByIds(Long[] ids)
    {
        return adHeartbeatDataMapper.deleteAdHeartbeatDataByIds(ids);
    }

    /**
     * 删除拉手心跳数据信息
     *
     * @param id 拉手心跳数据主键
     * @return 结果
     */
    @Override
    public int deleteAdHeartbeatDataById(Long id)
    {
        return adHeartbeatDataMapper.deleteAdHeartbeatDataById(id);
    }

    @Override
    public List<AdHeartbeatData> selectAdHeartbeatDataInfoList(AdHeartbeatData adHeartbeatData) {

        Calendar date = Calendar.getInstance();
        date.add(Calendar.DAY_OF_YEAR,-30);
        adHeartbeatData.setUpdateTime(date.getTime());
        List<AdHeartbeatData> list = adHeartbeatDataMapper.selectAdHeartbeatDataInfoList(adHeartbeatData);
        return list;
    }

    @Override
    public AdHeartbeatData selectAdHeartbeatDataByImei(String imei) {
        return adHeartbeatDataMapper.selectAdHeartbeatDataByImei(imei);
    }
}
