package com.ruoyi.advertiser.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.ruoyi.advertiser.constant.SubwayConstant;
import com.ruoyi.advertiser.domain.CarouselPlan;
import com.ruoyi.advertiser.domain.dto.*;
import com.ruoyi.advertiser.domain.req.CarouselPlanParamReq;
import com.ruoyi.advertiser.mapper.CarouselPlanMapper;
import com.ruoyi.advertiser.mapper.MaterialMapper;
import com.ruoyi.advertiser.service.ICarouselPlanService;
import com.ruoyi.advertiser.util.CreateCarouselPlanUtil;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.service.ISysConfigService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 广告轮播计划 Service业务层处理
 *
 * @author zhangzhanwei
 * @date 2021-09-24
 */
@Service
public class CarouselPlanServiceImpl implements ICarouselPlanService {
    @Resource
    private CarouselPlanMapper adCarouselPlanMapper;

    @Resource
    private RedisCache redisCache;

    @Resource
    private MaterialMapper materialMapper;

    @Resource
    private CreateCarouselPlanUtil createCarouselPlanUtil;

    @Resource
    private ISysConfigService configService;

    /**
     * 查询广告轮播计划 精确到列车
     *
     * @param carouselPlanParamVo
     * @return 广告轮播计划
     */
    @Override
    public AjaxResult selectAdCarouselPlanByAdCarouselPlanParamVo(CarouselPlanParamReq carouselPlanParamVo) {
        CarouselPlanDto adCarouselPlanDto = adCarouselPlanMapper.selectAdCarouselPlanByAdCarouselPlanParamVo(carouselPlanParamVo);
        if (adCarouselPlanDto == null) {
            return AjaxResult.error("暂无数据，请稍后查看！");
        }
        List<CarouselPlanDayPicsDto> list = getDayCarouselPlanData(adCarouselPlanDto,SubwayConstant.SUBWAY_ID,carouselPlanParamVo.getAdvertiseStartDate()).getDataList();
        // 封装返回的数据
        LinkedHashMap<String, List<CarouselPlanDayPicsDto>> mapData =
                list.stream().collect(Collectors.groupingBy(CarouselPlanDayPicsDto::getHour,LinkedHashMap::new,Collectors.toCollection(ArrayList::new)));
        return AjaxResult.success(mapData);
    }

    @Override
    public AjaxResult pushRedis(){
        createCarouselPlanUtil.createScreenAdvertiseTimeData();
        return AjaxResult.success();
    }

    /**
     * 说明：获取整日的图片轮播数据
     * @param adCarouselPlanDto
     * @return AdCarouselPlanDayDto
     */
    @Override
    public CarouselPlanDayDto getDayCarouselPlanData(CarouselPlanDto adCarouselPlanDto, Long subwayId, Date day){
        // 公益素材
        if (StringUtils.isEmpty(adCarouselPlanDto.getMorningPeakMaterialIds())) {
            adCarouselPlanDto.setMorningPeakMaterialIds(SubwayConstant.AD_PUBLIC_BENEFIT_MATERIAL);
        }
        // 图片素材
        Long[] morning = Convert.toLongArray(adCarouselPlanDto.getMorningPeakMaterialIds().split(";"));
        List<MaterialUrlDto> allPic = materialMapper.selectMaterialByIds(morning);
        // 计算地铁运营开始和结束时间
        Date stationStartDate = Convert.toDate(SubwayConstant.STATION_START_TIME);
        Date stationEndDate = Convert.toDate(SubwayConstant.STATION_END_TIME);
        // 图片刷新周期
        String sysDisplayTime = configService.selectConfigByKey("ad.muc.displayTime");
        int displayTime = StringUtils.isNotEmpty(sysDisplayTime)? Integer.valueOf(sysDisplayTime):SubwayConstant.MUC_DISPLAY_TIME;

        // 分钟数
        long morningBetweenMinute = DateUtil.between(stationStartDate, stationEndDate, DateUnit.MINUTE);
        // 生成图片列表
        int timeCount = (int) morningBetweenMinute / displayTime;
        List<MaterialUrlDto> morningTimeMaterials = new ArrayList<>();
        for (int i = 0; i < timeCount; i++) {
            morningTimeMaterials.add(allPic.get(i%allPic.size()));
        }

        // 处理图片轮询时间
        LinkedList<CarouselPlanDayPicsDto> list = new LinkedList<>();
        // 封装早高峰数据
        getTimeMapData(stationStartDate, list, morningBetweenMinute, morningTimeMaterials, 0, displayTime);

        CarouselPlanDayDto result = new CarouselPlanDayDto();
        result.setStationStartDate(stationStartDate)
                .setStationEndDate(stationEndDate)
                .setImageBindingIds(morning)
                .setDataList(list)
                .setAdMaterialsMorning(allPic);
       return result;
    }

    /**
     * 根据时间获取封装可展示的轮播数据
     *
     * @param startDate     时间段的开始时间
     * @param list       轮播数据
     * @param betweenMinute 时间差（分钟）
     * @param materials     图片素材集合
     * @param index         图片素材集合的下标
     * @param displayTime   图片轮播周期
     * @return
     */
    private int getTimeMapData(Date startDate, LinkedList<CarouselPlanDayPicsDto> list, long betweenMinute, List<MaterialUrlDto> materials, int index, int displayTime) {
        Date start = new Date(startDate.getTime());
        for (int i = 0; i < betweenMinute; i += displayTime, index++) {
            CarouselPlanDayPicsDto plan = new CarouselPlanDayPicsDto();
            Date end = new Date(start.getTime() + displayTime * 60 * 1000);
            plan.setHour(DateUtil.format(start, "HH") + ":00");
            plan.setTime(DateUtil.format(start, "HH:mm") + "~" + DateUtil.format(end, "HH:mm"));
            plan.setAdMaterialUrlDto(materials.get(index));
            list.add(plan);
            start = end;
        }
        return index;
    }

    /**
     * 根据拉手的imei和开始结束时间查询轮播计划
     *
     * @param imei        拉手imei
     * @param beginOfWeek 周开始时间
     * @param endOfWeek   周结束时间
     * @return
     */
    @Override
    public CarouselPlan selectAdCarouselPlanByImei(String imei, Date beginOfWeek, Date endOfWeek) {
        return adCarouselPlanMapper.selectAdCarouselPlanByImei(imei, beginOfWeek, endOfWeek);
    }

    @Override
    public CarouselPlanRedisDto getCarouselPlanCache(String key) {
        CarouselPlanRedisDto cacheObject = redisCache.getCacheObject(SubwayConstant.AD_CAROUSEL_PLAN_KEY + key);
        return cacheObject;
    }
}
