package com.ruoyi.advertiser.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IORuntimeException;
import cn.hutool.core.io.file.FileReader;
import cn.hutool.core.util.IdUtil;
import com.ruoyi.advertiser.constant.McuSignatureConstant;
import com.ruoyi.advertiser.constant.SeSignature;
import com.ruoyi.advertiser.domain.Material;
import com.ruoyi.advertiser.enums.MaterialStatusEnum;
import com.ruoyi.advertiser.mapper.MaterialMapper;
import com.ruoyi.advertiser.service.IMaterialService;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.setsm.service.SeActivatingService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.HttpClientErrorException;

import javax.annotation.Resource;
import javax.imageio.stream.FileImageOutputStream;
import java.io.File;
import java.util.List;

/**
 * 广告素材 Service业务层处理
 *
 * @author zhangzhanwei
 * @date 2021-09-13
 */
@Service
@Slf4j
public class MaterialServiceImpl implements IMaterialService {
    
    @Resource
    private MaterialMapper materialMapper;

    @Resource
    private SeActivatingService activatingService;

    @Override
    public List<Material> selectMaterialList(Material material) {
        List<Material> materialList = materialMapper.selectMaterialList(material);
        for (Material pic : materialList) {
            pic.setStatusDesc(MaterialStatusEnum.findDescByCode(pic.getStatus()));
        }
        return materialList;
    }

    @Override
    public int insertMaterial(Material material) {
        material.setStatus(MaterialStatusEnum.WAIT_SUBMIT.getCode());
        material.setCreateBy(SecurityUtils.getUsername());
        material.setCreateTime(DateUtils.getNowDate());
        return materialMapper.insertMaterial(material);
    }

    @Override
    public Material selectMaterialById(Long id) {
        return materialMapper.selectMaterialById(id);
    }

    @Override
    public int updateMaterial(Material material) {
        material.setUpdateBy(SecurityUtils.getUsername());
        material.setUpdateTime(DateUtils.getNowDate());
        return materialMapper.updateMaterial(material);
    }

    @Override
    public int deleteMaterialByIds(Long[] ids) {
        return materialMapper.deleteMaterialByIds(ids);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult passMaterial(Material picId) {
        Material material = materialMapper.selectMaterialById(picId.getId());
        if (material == null || "".equals(material.getConvertedFileUrl())){
            return AjaxResult.error("签名失败！未查询到图片信息。");
        }

        String tscUrl = RuoYiConfig.getProfile()+material.getConvertedFileUrl().substring(material.getConvertedFileUrl().indexOf("/upload"));
        File file;
        FileReader fileReader;
        try {
            file = new File(tscUrl);
            fileReader = new FileReader(file);
        }catch (IORuntimeException e){
            return AjaxResult.error("签名失败！未读取到文件，请重新上传文件。");
        }
        byte[] infoImage = fileReader.readBytes();
        byte[] temp = new byte[infoImage[McuSignatureConstant.SIGNATURE_LENGTH_INDEX]];
        System.arraycopy(infoImage, McuSignatureConstant.SIGNATURE_DATA_INDEX, temp, 0, temp.length);

        //应用方流水号
        String requestId = IdUtil.simpleUUID();
        String time = DateUtil.format(DateUtil.date(), "yyyyMMddHHmmss");
        SeSignature seSignature = new SeSignature();
        seSignature.setApTxnNo(requestId).setSignType("01").setDataForSign(Convert.toHex(temp)).setTimes(time);
        String signature;
        try {
            signature = activatingService.signature(requestId, seSignature);
        }catch (HttpClientErrorException.BadRequest e){
            log.error(e.getMessage());
            return AjaxResult.error("签名失败！复旦微签名接口连接错误，请联系管理员。");
        }catch (NullPointerException e){
            log.error(e.getMessage());
            return AjaxResult.error("签名失败！复旦微签名接口返回信息为空，请联系管理员。");
        }catch (ServiceException e){
            log.error(e.getMessage());
            return AjaxResult.error("签名失败！复旦微签名接口出现其他未知异常，请联系管理员。");
        }
        if ("".equals(seSignature)){
            return AjaxResult.error("签名失败！复旦微签名接口返回信息为空，请联系管理员。");
        }

        byte[] signResult = Convert.hexToBytes(signature);
        System.arraycopy(signResult, 0, infoImage, McuSignatureConstant.SIGNATURE_DATA_INDEX, signResult.length);
        infoImage[McuSignatureConstant.IS_SIGNATURE_INDEX] = McuSignatureConstant.YES_SIGNATURE;
        infoImage[McuSignatureConstant.SIGNATURE_LENGTH_INDEX] = (byte) signResult.length;

        material.setConvertedFileUrl(material.getConvertedFileUrl().replace(material.getConvertedFileUrl()
                .substring(material.getConvertedFileUrl().lastIndexOf("/") +1), material.getId() + ".LSC"));
        try {
            FileImageOutputStream imageOutput = new FileImageOutputStream(file);
            imageOutput.write(infoImage, 0, infoImage.length);
            imageOutput.close();
            file.renameTo(new File(RuoYiConfig.getProfile()+material.getConvertedFileUrl().substring(material.getConvertedFileUrl().indexOf("/upload"))));
        }catch (Exception e){
            log.error(e.getMessage());
            return AjaxResult.error("签名失败！保存签名文件出错。");
        }
        Material updatedMaterial = new Material();
        updatedMaterial.setId(picId.getId());
        updatedMaterial.setConvertedFileUrl(material.getConvertedFileUrl());
        updatedMaterial.setStatus(MaterialStatusEnum.PASS.getCode());
        materialMapper.updateMaterial(updatedMaterial);
        return AjaxResult.success("签名成功！");
    }
}
