package com.ruoyi.advertiser.util;

import cn.hutool.core.util.ArrayUtil;
import com.ruoyi.advertiser.constant.SubwayConstant;
import com.ruoyi.advertiser.domain.CarouselPlan;
import com.ruoyi.advertiser.domain.Material;
import com.ruoyi.advertiser.domain.dto.*;
import com.ruoyi.advertiser.enums.MaterialStatusEnum;
import com.ruoyi.advertiser.mapper.CarouselPlanMapper;
import com.ruoyi.advertiser.service.ICarouselPlanService;
import com.ruoyi.advertiser.service.IMaterialService;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 生产轮播计划数据的定时任务
 *
 * @author zhangzhanwei
 * @date 2021-09-14
 */

@Component("carouselPlanTask")
@Slf4j
public class CreateCarouselPlanUtil {

    @Resource
    private CarouselPlanMapper carouselPlanMapper;

    @Resource
    private RedisCache redisCache;

    @Resource
    private ICarouselPlanService adCarouselPlanService;

    @Resource
    private IMaterialService materialService;

    /**
     * 创建轮播计划数据
     */
    @Transactional(rollbackFor = Exception.class)
    public void createScreenAdvertiseTimeData() {
        log.info("开始生产轮播计划数据.................");
        Date startTime = DateUtils.dateTime(DateUtils.YYYY_MM_DD, DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD,DateUtils.getNowDate()));
        Date endTime = startTime;
        // 生成数据时先删除计划表里的数据
        carouselPlanMapper.deleteAdCarouselPlanByTime(startTime, endTime);
        // 要生成的轮播
        CarouselPlan carouselPlan = createCarouselPlanDataBySubwayLine(startTime);
        // 放到redis里
        setRedisPan(carouselPlan,startTime);
        log.info("生产轮播计划数据结束.................");
    }

    /**
     * @param thatDay 当前日期
     * @return CarouselPlan 轮播计划
     */
    public CarouselPlan createCarouselPlanDataBySubwayLine(Date thatDay) {
        CarouselPlan carouselPlan = new CarouselPlan();
        carouselPlan.setLineId(SubwayConstant.SUBWAY_ID);
        carouselPlan.setSubwayId(SubwayConstant.SUBWAY_ID);
        carouselPlan.setAdvertiseStartDate(thatDay);
        carouselPlan.setAdvertiseEndDate(thatDay);
        List<Material> picList = materialService.selectMaterialList(new Material().setPicShow(true).setStatus(MaterialStatusEnum.PASS.getCode()));
        if (picList.size()>0){
            String picIds = picList.stream().map(Material::getId).map(id-> Convert.toStr(id)).collect(Collectors.joining(";"));
            carouselPlan.setMorningPeakMaterialIds(picIds);
        } else {
            // 公益素材
            carouselPlan.setMorningPeakMaterialIds(SubwayConstant.AD_PUBLIC_BENEFIT_MATERIAL);
        }
        carouselPlan.setCreateBy("admin");
        carouselPlan.setCreateTime(DateUtils.getNowDate());
        carouselPlanMapper.insertAdCarouselPlan(carouselPlan);
        return  carouselPlan;
    }

    /**
     * 封装数据到redis-没有轮播图的列车要播放公益图片
     *
     * @param carouselPlan
     */
    public void setRedisPan(CarouselPlan carouselPlan, Date time) {
        // 封装生成参数
        CarouselPlanDto adCarouselPlanDto = new CarouselPlanDto();
        adCarouselPlanDto.setMorningPeakMaterialIds(carouselPlan.getMorningPeakMaterialIds());
        // 获取轮播数据
        CarouselPlanDayDto dayDto = adCarouselPlanService.getDayCarouselPlanData(adCarouselPlanDto, carouselPlan.getSubwayId(), time);
        // 处理图片集合
        List<MaterialLscUrlDto> adMaterialsMorning = dayDto.getAdMaterialsMorning().stream().map(adMaterialUrlDto->{
            MaterialLscUrlDto adMaterialLscUrlDto = new MaterialLscUrlDto();
            adMaterialLscUrlDto.setId(adMaterialUrlDto.getId());
            adMaterialLscUrlDto.setConvertedFileUrl(adMaterialUrlDto.getConvertedFileUrl());
            return adMaterialLscUrlDto;
        }).collect(Collectors.toList());

        // 处理图片轮播id数据集合
        List<CarouselPlanDayPicsDto> dateList = dayDto.getDataList();
        Long[] imageIds = dateList.stream().map(item->item.getAdMaterialUrlDto().getId()).toArray(Long[]::new);

        // 不管有没有图片，都要把1，2，3公益图片放进去
        Long[] bindingIds = dayDto.getImageBindingIds();
        if (!ArrayUtil.contains(bindingIds, Long.valueOf(SubwayConstant.AD_PUBLIC_BENEFIT_MATERIAL))){
            bindingIds = ArrayUtil.insert(bindingIds,0, new Long[]{ Long.valueOf(SubwayConstant.AD_PUBLIC_BENEFIT_MATERIAL) });
        }

        // 存入redis
        CarouselPlanRedisDto adCarouselPlanRedisDto = new CarouselPlanRedisDto();
        adCarouselPlanRedisDto.setAdMaterialsMorning(adMaterialsMorning)
                .setAdMaterialsEvening(adMaterialsMorning)
                .setAdMaterialsNormal(adMaterialsMorning)
                .setImageIds(imageIds)
                .setStationStartDate(dayDto.getStationStartDate())
                .setStationEndDate(dayDto.getStationEndDate())
                .setImageBindingIds(bindingIds);
        redisCache.setCacheObject(SubwayConstant.AD_CAROUSEL_PLAN_KEY + carouselPlan.getSubwayId().toString(), adCarouselPlanRedisDto);

    }

}
