package com.ruoyi.mcu.common;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IORuntimeException;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.mcu.server.DealChannel;
import com.ruoyi.setsm.exception.McuServiceException;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.nio.channels.*;
import java.nio.channels.spi.SelectorProvider;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * @author wzsdch_p751
 * @date 2022/07/14 17:22
 **/
@Slf4j(topic = "sys-mcu")
public class EventLoop implements Runnable{
    private final Selector selector;

    private Thread thread;
    private int queueNum=16;
    private LinkedBlockingQueue<Runnable> taskQueue;

    public EventLoop() throws IOException {
        taskQueue = new LinkedBlockingQueue<>(queueNum);
        this.selector = SelectorProvider.provider().openSelector();
        this.thread = new Thread(this);
        this.thread.start();
    }


    public void register(SocketChannel socketChannel, int keyOps, List<DealChannel> dealChannelList) throws InterruptedException {
        log.info("taskQueue数量：{},thread:{},this.Thread:{}  ****************************************************",taskQueue.size(),thread.getName(),this.thread.getName());
        //log.info("taskQueue11111");
        //放入taskQueue，并没有在这里执行下端代码，在下面调用task.run()中执行
        taskQueue.put(()->{
            try {
                //log.info("taskQueue33333");
                //DealChannel dealChannel = new DealChannel(socketChannel);
                DealChannel dealChannel = SpringUtils.getBean("dealChannel");
                dealChannel.init(socketChannel,selector);
                SelectionKey selectionKey = socketChannel.register(selector, keyOps);
                selectionKey.attach(dealChannel);
                dealChannelList.add(dealChannel);
            } catch (ClosedChannelException e) {
                e.printStackTrace();
            }
        });
        //log.info("taskQueue22222");
        //唤起selector上阻塞的线程
        selector.wakeup();
    }

    @Override
    public void run() {
        while (!Thread.currentThread().isInterrupted()){
            try {
                //log.info("等待select执行,taskQueue数量：{}，线程：{},{}****************************************************",taskQueue.size(),Thread.currentThread().getId(),Thread.currentThread().getName());
                int eventNum = selector.select();
                //log.info("正在select运行,taskQueue数量：{}，线程：{},{}****************************************************",taskQueue.size(),Thread.currentThread().getId(),Thread.currentThread().getName());
                //1.eventNum>0如果有事件说明就是读写事件；2.eventNum=0如果没有事件，就是添加新的taskQueue对象事件。
                if(eventNum>0) {
                    Iterator<SelectionKey> keyIterator = selector.selectedKeys().iterator();
                    while (keyIterator.hasNext()) {
                        SelectionKey selectionKey = keyIterator.next();
                        keyIterator.remove();

                        //获取处理Channel
                        DealChannel dealChannel = (DealChannel) selectionKey.attachment();

                        String remoteAddress = "";
                        SocketChannel socketChannel = null;
                        try {
                            //获取channel
                            SocketChannel channel = dealChannel.getChannel();
                            if(channel!=null){
                            remoteAddress = channel.getRemoteAddress().toString();
                            }

                            //远程突然无条件断开连接
                            if (!selectionKey.isValid()) {
                                log.info("时间: {} ，设备IP：{} 断开连接5 <br/>", DateUtil.now(), remoteAddress);
                                break;
                            }
                            //log.info("设备正在收发消息中...  <br/>");
                            // read 事件
                            if (selectionKey.isReadable()) {
                                dealChannel.setLastReadDate(System.currentTimeMillis());
                                dealChannel.read(selectionKey, remoteAddress);
                            }
                            //回复数据
                            if (selectionKey.isWritable()) {
                                //log.info("正在发送数据中...设备ip:{}",remoteAddress);
                                dealChannel.write(selectionKey);
                                log.info("数据发送完成！设备ip:{}\r\n--------------------------------------------------------------------------------",
                                        remoteAddress);
                            }
                        } catch (CancelledKeyException e) {
                            log.info("时间: {} ，设备IP：{} 断开连接2 <br/>", DateUtil.now(), remoteAddress);
                            selectionKey.cancel();
                            break;
                        } catch (McuServiceException e) {
                            log.info(e.toString());
                            log.info("发生错误：" + e.getMessage());
                            selectionKey.cancel();
                            break;
                        } catch (IOException e) {
                            log.info(e.toString());
                            log.info("时间: {} ，设备IP：{} 断开连接3:IOException <br/>", DateUtil.now(), remoteAddress);

                            selectionKey.cancel();
                            //socketChannel.close();
                            break;
                        } catch (IORuntimeException e) {
                            log.info(e.toString());
                            log.info("时间: {} ，设备IP：{} 断开连接4:IORuntimeException：未找到文件 <br/>", DateUtil.now(), remoteAddress);
                            selectionKey.cancel();
                            break;
                        }
                        catch (NullPointerException e) {
                            log.info(e.toString());
                            log.info("时间: {} ，设备IP：{} 断开连接10:Exception：未知错误 <br/>", DateUtil.now(), remoteAddress);
                            selectionKey.cancel();
                            break;
                        }catch (Exception e) {
                            log.info(e.toString());
                            log.info("时间: {} ，设备IP：{} 断开连接99:Exception：未知错误 <br/>", DateUtil.now(), remoteAddress);
                            selectionKey.cancel();
                            break;
                        }
                    }
                }
                Runnable task;
                while ((task = taskQueue.poll()) != null) {
                    task.run();
                    //log.info("taskQueue44444");
                }

            }catch (IOException e){
                e.printStackTrace();
                log.error("发生错误:"+e);
            }
        }
    }
}
