package com.ruoyi.mcu.common;

import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.mcu.config.McuSocketConfig;
import com.ruoyi.mcu.server.DealChannel;
import java.io.IOException;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.SocketChannel;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author wzsdch_p751
 * @date 2022/07/18 13:09
 **/
public class EventLoopGroup {



    private EventLoop[] eventLoops;


    private List<DealChannel> dealChannelList;

    private final AtomicInteger idx = new AtomicInteger(0);

    public EventLoopGroup() throws IOException {
        //创建事件池
        McuSocketConfig mcuSocketConfig = SpringUtils.getBean("mcuSocketConfig");
        this.eventLoops = new EventLoop[mcuSocketConfig.getDealThreadQuantity()];
        for (int i = 0; i < eventLoops.length; i++) {
            eventLoops[i] = new EventLoop();
        }
        dealChannelList = new LinkedList<>();
    }

    public EventLoop[] getEventLoops() {
        return eventLoops;
    }

    public EventLoop next(){
        //轮询处理，指向下一个事件类
        int nextint = idx.getAndIncrement() & eventLoops.length-1;
        return eventLoops[nextint];
    }

    /**
     * 注册事件处理
     * @param socketChannel
     * @param keyOps
     * @throws ClosedChannelException
     */
    public void  register(SocketChannel socketChannel, int keyOps) throws InterruptedException {
        next().register(socketChannel,keyOps,dealChannelList);
    }

    public List<DealChannel> getDealChannelList() {
        return dealChannelList;
    }
}
