package com.ruoyi.mcu.reconstruct.tag;

import cn.hutool.core.convert.Convert;
import com.google.common.primitives.Bytes;
import com.ruoyi.mcu.config.McuSm4KeyConfig;
import com.ruoyi.mcu.constant.McuBeatPackHeadAndTailConstant;
import com.ruoyi.mcu.domain.AdHeartBeatPackage;
import com.ruoyi.mcu.util.CRC16ModbusUtil;
import com.ruoyi.mcu.util.DataTransitionUtil;
import com.ruoyi.mcu.util.SM4Util;
import lombok.Data;
import lombok.experimental.Accessors;

import java.io.Serializable;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

/**
 * 标签基类（TLV结构）
 */
@Data
@Accessors(chain = true)
public class BaseTag implements Serializable {
    /** 标签类型 */
    protected byte tagType;

    /** 标签长度 */
    protected int tagLength;

    /** 标签值 */
    protected byte[] tagValue;

    public static byte[] reply(AdHeartBeatPackage beatPack, byte identification, byte packageType, List<BaseTag> tagList) {
        List<Byte> packageData = new LinkedList<>();
        // 包头
        packageData.add(McuBeatPackHeadAndTailConstant.PACKAGE_HEAD);
        // 唯一编码 IMEI
        Arrays.stream(beatPack.getIMEI()).forEach(imeiData -> packageData.add((byte) imeiData));
        // 协议版本
        packageData.add((byte) 0x10);
        // 设备类型
        Arrays.stream(beatPack.getEquipmentType()).forEach(equipmentTypeData -> packageData.add((byte) equipmentTypeData));
        // 包序号
        addTwoByte(packageData, beatPack.getPackageSeq());
        // 标识
        packageData.add(identification);
        // 包类型
        packageData.add(packageType);
        // 标签数量
        packageData.add((byte) tagList.size());
        // 消息体总长度 标签类型1字节 + 标签长度2字节
        // 标签值的长度不确定，在循环后可确定
        int msgBodyLength = tagList.size() * 3;
        int msgBodyLengthIndex = packageData.size();
        addTwoByte(packageData, -1);
        // 消息体
        for (BaseTag tag : tagList) {
            // t
            packageData.add(tag.getTagType());
            // l
            addTwoByte(packageData, tag.getTagLength());
            // 计算消息体总长度
            msgBodyLength += tag.getTagLength();
            // v
            packageData.addAll(Bytes.asList(tag.getTagValue()));
        }
        // 循环结束后，重新设置消息体总长度
        setTwoByte(packageData, msgBodyLengthIndex, msgBodyLength);
        // crc16校验
        long crc16 = CRC16ModbusUtil.getValue(Bytes.toArray(packageData));
        addTwoByte(packageData, (int) crc16);
        // 包尾
        packageData.add(McuBeatPackHeadAndTailConstant.PACKAGE_TAIL);

        byte[] res = Bytes.toArray(packageData);

        // bit3-bit0 表示消息体加密方式 0 为不加密，其它值会对应不同的加密方法
        if ((beatPack.getIdentification() & 0x01) == 0x01) {
            // 消息体数据
            List<Byte> msgBody = packageData.subList(msgBodyLengthIndex, msgBodyLengthIndex + msgBodyLength);
            // sm4加密
            byte[] encryptMsgBody = SM4Util.payloadEncrypt(Bytes.toArray(msgBody), Convert.hexToBytes(McuSm4KeyConfig.SM4KEY),
                    DataTransitionUtil.intsToBytes(beatPack.getIMEI()), beatPack.getPackageSeq());
            System.arraycopy(encryptMsgBody, 0, res, msgBodyLengthIndex, msgBodyLength);
        }

        return res;
    }

    /**
     * 设置2个字节长度数据
     */
    private static void setTwoByte(List list, int index, int length) {
        list.set(index, (byte) ((length >> 8) & 0xff));
        list.set(index + 1, (byte) (length & 0xff));
    }

    /**
     * 增加2个字节长度数据
     */
    private static void addTwoByte(List list, int length) {
        list.add((byte) ((length >> 8) & 0xff));
        list.add((byte) (length & 0xff));
    }

}
