package com.ruoyi.mcu.server;


import cn.hutool.core.date.DateUtil;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.mcu.common.EventLoopGroup;
import com.ruoyi.mcu.config.McuSocketConfig;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.util.Iterator;

/**
 * @author zzw
 * @date 2021/6/17
 * @Description: 服务器
 */
@Component
@Slf4j(topic = "sys-mcu")
@NoArgsConstructor
public class ControlServer implements ApplicationRunner {

    //单例
    public static ControlServer controlServer;
    public static ControlServer getInstence() {
        if (controlServer == null) {
            controlServer = new ControlServer();
        }
        return controlServer;
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        ControlServer controlServer = ControlServer.getInstence();
        controlServer.init();
    }

    private final Charset charset = Charset.forName("UTF-8");

    //记录连接对象的容器（里面包含了该连接的全部信息）
    //private static List<ChannelBean> list = new ArrayList<>();

    /**
     * 初始化
     *
     * @throws IOException
     */
    public void init(){
        // 创建服务器
        try (ServerSocketChannel server = ServerSocketChannel.open()){
            McuSocketConfig mcuSocketConfig = SpringUtils.getBean(McuSocketConfig.class);
            // 获取端口
            int port = mcuSocketConfig.getSocketPort();
            // 获取缓冲区大小
            //缓冲区大小
            int bufferSize = mcuSocketConfig.getBufferSize();
            // 绑定监听端口
            server.bind(new InetSocketAddress(port));
            //非阻塞的方式
            server.configureBlocking(false);
            // 创建 selector 对象
            Selector selector = server.provider().openSelector();
            //注册到选择器上，设置为监听状态
            // 只关注accept事件 （accept 事件 - 会在有连接请求时触发）
            server.register(selector, SelectionKey.OP_ACCEPT);
            //EventLoop eventLoop = SpringUtils.getBean("eventLoop");
            //创建一组事件查询器
            EventLoopGroup eventLoopGroup = SpringUtils.getBean("eventLoopGroup");

            log.info("系统已就绪等待mcu设备连接。。。<br/>");
            while (true) {
                // select() 没有事件发生线程阻塞，有事件发生线程才会恢复运行
                // select 事件未处理时 不会阻塞
                selector.select();
                // 获取所有连接（处理事件，selectedKeys内包含了所有发生的时间）
                // selector会在发生事件后，向selectedKeys加入key，但不会删除
                Iterator<SelectionKey> keyIterator = selector.selectedKeys().iterator();
                while (keyIterator.hasNext()) {
                    SelectionKey sk = keyIterator.next();
                    // 处理key时，要从selectedKeys集合汇总删除，否则下次处理就会出空指针
                    keyIterator.remove();

                    if(sk.isAcceptable()){
                        ServerSocketChannel ssc = (ServerSocketChannel) sk.channel();
                        SocketChannel socketChannel = ssc.accept();
                        socketChannel.configureBlocking(false);
                        log.info("时间：" + DateUtil.now() + ", 发现一个新设备连接，IP为:" + socketChannel.getRemoteAddress().toString().substring(1) + "<br/>");

                        //注册设备读取事件
                        eventLoopGroup.register(socketChannel,SelectionKey.OP_READ);

                    }
                    // 处理一个连接（事件） 如果不处理会变成非阻塞状态
                    //dealWithSelectionKey(server,sk);
                }
            }
        }catch (Exception e){
            log.error("init error", e);
        }
    }

}
