package com.ruoyi.mcu.server;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.file.FileReader;
import cn.hutool.core.util.IdUtil;
import com.ruoyi.advertiser.constant.SubwayConstant;
import com.ruoyi.advertiser.domain.AdFirmwareUpgrade;
import com.ruoyi.advertiser.domain.AdHandle;
import com.ruoyi.advertiser.domain.AdHeartbeatData;
import com.ruoyi.advertiser.domain.dto.CarouselPlanRedisDto;
import com.ruoyi.advertiser.domain.dto.MaterialLscUrlDto;
import com.ruoyi.advertiser.service.*;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.mcu.config.McuSm4KeyConfig;
import com.ruoyi.mcu.constant.*;
import com.ruoyi.mcu.domain.AdHeartBeatPackage;
import com.ruoyi.mcu.domain.AdMessageBody;
import com.ruoyi.mcu.reconstruct.tag.BaseTag;
import com.ruoyi.mcu.reconstruct.tag.FirmwareUpgradeStatusTag;
import com.ruoyi.mcu.reconstruct.tag.HardwareVersionTag;
import com.ruoyi.mcu.reconstruct.tag.ImageTransmitStatusTag;
import com.ruoyi.mcu.util.CRC16ModbusUtil;
import com.ruoyi.mcu.util.DataTransitionUtil;
import com.ruoyi.mcu.util.SM4Util;
import com.ruoyi.setsm.domain.SeActivating;
import com.ruoyi.setsm.service.SeActivatingService;
import com.ruoyi.system.service.ISysConfigService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.util.*;

/**
 * @author wzsdch_p751
 * @date 2022/07/15 09:35
 **/
@Service
@Slf4j(topic = "sys-mcu")
public class DealServer {

    @Resource(name="adHandleServiceImpl")
    private  IAdHandleService adHandleService;

    @Resource(name="carouselPlanServiceImpl")
    private ICarouselPlanService adCarouselPlanService;

    @Resource(name="seActivatingServiceImpl")
    private SeActivatingService seActivatingService;

    @Resource(name="adHeartbeatDataServiceImpl")
    private IAdHeartbeatDataService adHeartbeatDataService;

    @Resource
    private ISysConfigService configService;

    /**
     * 处理心跳包
     * @param beatPack
     * @return
     */
    public byte[] upBeatReply(AdHeartBeatPackage beatPack) {
        String imei=beatPack.getImeiStr();

        //如果数据库没有拉手数据，则不回复心跳。忽略该条心跳数据
        AdHandle adHandleLocal = adHandleService.selectAdHandleByImei(imei);
        if(adHandleLocal==null){
            log.info("数据库没有此拉手数据：" + imei);
            return null;
        }
        AdHandle adHandle=null;
        try {
            // 存储心跳数据
            AdHeartbeatData adHeartbeatData = decodingAdHeartbeatData(beatPack.getDeviceInfo());
            // 不需要保存首位的0
            adHeartbeatData.setImei(imei);
            //存入数据库
            adHeartbeatDataService.insertAdHeartbeatData(adHeartbeatData);
            // 根据心跳修改拉手信息
            adHandle = new AdHandle();
                    // 15位的imei
            adHandle.setImei(adHeartbeatData.getImei());
            adHandle.setIccid(adHeartbeatData.getIccid());
            adHandle.setOnlineStatus("1");
            adHandle.setSoftwareVersion(adHeartbeatData.getSw());
            adHandle.setHardwareVersion(adHeartbeatData.getHw());
            adHandle.setUpdateTime(new Date());
            try {
                adHandleService.updateHandleByHeartbeat(adHandle);
            }catch(Exception e2){
                log.error("更新拉手信息发生错误：adHandle:{}",adHandle);
            }
        } catch (Exception e) {
            log.error("数据转换异常：", e);
        }

        log.info("已接收到MCU发来的普通心跳包，开始解析并回复心跳包。。。<br/>");

        //获取轮播计划
        CarouselPlanRedisDto carouselPlanCache = getcarouselPlanCache(imei);
        Integer[] imageSeq;
        if(carouselPlanCache==null){
            imageSeq= new Integer[0];
        }else {
            imageSeq = DataTransitionUtil.longsToIntegers(carouselPlanCache.getImageIds());
        }

        int imageNum = imageSeq.length; // 图片数量
        int len = 4 * imageNum; // 图片序号长度
        byte[] info = new byte[len + 3000];
        int i = 0; // info数组下标
        // 包头
        info[i++] = McuBeatPackHeadAndTailConstant.PACKAGE_HEAD;
        // IMEI
        byte[] imeiByte = DataTransitionUtil.intsToBytes(beatPack.getIMEI());
        System.arraycopy(imeiByte, 0, info, i, imeiByte.length);
        i += imeiByte.length;
        info[i++] = 0x10; // 协议版本
        // 设备类型
        byte[] equipmentType = DataTransitionUtil.intsToBytes(beatPack.getEquipmentType());
        System.arraycopy(equipmentType, 0, info, i, equipmentType.length);
        i += equipmentType.length;
        // 包序号
        info[i++] = (byte) ((beatPack.getPackageSeq() >> 8) & 0xff);
        info[i++] = (byte) (beatPack.getPackageSeq() & 0xff);
        // 标识
        info[i++] = McuBeatIdentificationConstant.BEAT_IDENTIFICATION_0X00;
        // 包类型
        info[i++] = McuBeatPackTypeConstant.DOWN_BEAT_PACKAGE_TYPE;
        // 心跳包回复标签数量
        info[i++] = McuOtherConstant.BEAT_PACKAGE_REPLY_LABEL_NUM;
        // 消息体长度
        int msgLen = len + McuOtherConstant.BEAT_PACKAGE_REPLY_ELSE_LENGTH;
        int msgLenIndex = i; //消息体长度下标
        info[i++] = (byte) ((msgLen >> 8) & 0xff);
        info[i++] = (byte) ((msgLen) & 0xff);
        // 根据屏幕类型、硬件版本查询最新的固件包
        AdFirmwareUpgrade adFirmwareUpgrade = null;
        // 硬件版本
        info[i++] = McuBeatPackTagConstant.TAG_DEV_HW_REV;
        info[i++] = 0x00;
        info[i++] = 0x01;
        info[i++] = 0x10;
        // 软件版本：如果没有固件包，就不传
        if (adFirmwareUpgrade != null) {
            info[i++] = McuBeatPackTagConstant.TAG_DEV_SW_REV;
            info[i++] = 0x00;
            info[i++] = 0x01;
            String softwareVersion = adFirmwareUpgrade.getMajorVersion().toString() + adFirmwareUpgrade.getSubVersion().toString();
            info[i++] = Convert.hexToBytes(softwareVersion)[0];
        }
        // 心跳周期
        String sysHbiTime = configService.selectConfigByKey("ad.mcu.HBI");
        int hbiTime = StringUtils.isNotEmpty(sysHbiTime)? Integer.valueOf(sysHbiTime):SubwayConstant.MUC_HBI;
        info[i++] = McuBeatPackTagConstant.TAG_DEV_HBI;
        info[i++] = 0x00;
        info[i++] = 0x02;
        info[i++] = (byte) ((hbiTime >> 8) & 0xff);
        info[i++] = (byte) ((hbiTime) & 0xff);
        // 图片刷新周期
        String sysDisplayTime = configService.selectConfigByKey("ad.muc.displayTime");
        int displayTime = StringUtils.isNotEmpty(sysDisplayTime)? Integer.valueOf(sysDisplayTime):SubwayConstant.MUC_DISPLAY_TIME;
        info[i++] = McuBeatPackTagConstant.TAG_DEV_DISIMAGE_CYCLE;
        info[i++] = 0x00;
        info[i++] = 0x02;
        info[i++] = (byte) ((displayTime >> 8) & 0xff);
        info[i++] = (byte) ((displayTime) & 0xff);

        // 服务器端设备绑定图片列表
        info[i++] = McuBeatPackTagConstant.TAG_DEV_SERVER_IMAGE_INFO;
        Integer[] imageBindingIds = carouselPlanCache==null?new Integer[0]:DataTransitionUtil.longsToIntegers(carouselPlanCache.getImageBindingIds());
        int bindiImageNum = imageBindingIds.length; // 图片数量
        int lenTag0E = bindiImageNum * 4 + 2;
        info[i++] = (byte) ((lenTag0E >> 8) & 0xff);
        info[i++] = (byte) ((lenTag0E) & 0xff);

        info[i++] = (byte) ((bindiImageNum >> 8) & 0xff);
        info[i++] = (byte) ((bindiImageNum) & 0xff);

        for (int j = 0; j < bindiImageNum; j++) {
            if (imageBindingIds[j] != null) {
                i = getImageList(imageBindingIds, info, i, j);
            }
        }

        // mcu轮播开始时间
        String carouselBeginTimeText = carouselPlanCache==null?"000000":DateUtil.format(carouselPlanCache.getStationStartDate(), "HHmmss");
        char[] carouselBeginTime = carouselBeginTimeText.toCharArray();
        // mcu轮播结束时间
        String carouselEndTimeText = carouselPlanCache==null?"000000":DateUtil.format(carouselPlanCache.getStationEndDate(), "HHmmss");
        char[] carouselEndTime = carouselEndTimeText.toCharArray();
        // mcu轮播开始和结束时间
        info[i++] = McuBeatPackTagConstant.TAG_DEV_OPEN_DEVTIME;
        info[i++] = 0x00;
        info[i++] = 0x06;
        i = getCarouselTime(info, i, carouselBeginTime);
        i = getCarouselTime(info, i, carouselEndTime);
        // 设置显示捆绑列表
        info[i++] = McuBeatPackTagConstant.TAG_DEV_IMAGE_LIST;
        int imageListLenIndex = i; // 图片列表长度下标
        info[i++] = 0;
        info[i++] = 0;
        if (len != 0) {
            int startTime = ((carouselBeginTime[0] - 0x30) * 10 + (carouselBeginTime[1] - 0x30)) * 60 + ((carouselBeginTime[2] - 0x30) * 10 + (carouselBeginTime[3] - 0x30));
            int stopTime = ((carouselEndTime[0] - 0x30) * 10 + (carouselEndTime[1] - 0x30)) * 60 + ((carouselEndTime[2] - 0x30) * 10 + (carouselEndTime[3] - 0x30));
            if (startTime > stopTime) {
                stopTime += (24 * 60);
            }
            int countDisp = startTime;
            int imageCn = 0;
            int mCnTimes = 0;
            while (countDisp < stopTime) {
                i = getImageList(imageSeq, info, i, imageCn);
                countDisp += displayTime;
                if (imageCn < (imageNum - 1)) {
                    imageCn++;
                } else {
                    imageCn = 0;
                }
                mCnTimes++;
            }

            info[imageListLenIndex++] = (byte) ((4 * mCnTimes >> 8) & 0xff);
            info[imageListLenIndex++] = (byte) ((4 * mCnTimes) & 0xff);
        }
        info[msgLenIndex++] = (byte) (((i - 19) >> 8) & 0xff);
        info[msgLenIndex++] = (byte) (((i - 19)) & 0xff);

        byte[] crc16Data = getCrc16Data(i, info, beatPack, msgLenIndex);
        return crc16Data;
    }

    private AdHeartbeatData decodingAdHeartbeatData(AdMessageBody messageBody) {
        AdHeartbeatData obj = new AdHeartbeatData();
        obj.setCsq(messageBody.getCsq());
        obj.setPci(messageBody.getPci());
        obj.setIccid(messageBody.getIccid());

        // 0x10 => V1.0
        obj.setHw(Integer.parseInt(Integer.toHexString(messageBody.getHw())));
        obj.setSw(Integer.parseInt(Integer.toHexString(messageBody.getSw())));

        obj.setHbiTime(messageBody.getHbiTime());
        obj.setPowerVol(messageBody.getPowerVol());

        // 16进制字符串 => 对应的ASCII码
        String location = messageBody.getDevLocation();
        if (StringUtils.isNotBlank(location)) {
            StringBuilder locationStr = new StringBuilder();
            for (int i = 0; i < location.length(); i+=2) {
                String hex = location.substring(i, i + 2);
                char ascii = (char) Integer.parseInt(hex, 16);
                locationStr.append(ascii);
            }
            obj.setDevLocation(locationStr.toString());
        }

        obj.setErrorCode(messageBody.getErrorCode());
        //当错误码8时，不算做异常20220908

        obj.setStatus(calculateStatus(obj.getErrorCode()));
        obj.setDisplayTime(messageBody.getDisplayTime());
        obj.setLoraFrq(messageBody.getLoraFrq());
        obj.setResidueSize(messageBody.getDiskIoTotalSize() - messageBody.getDiskIoUserSize());

        //回复数据中带的图片信息 例："211-2,202-1,223-4"
        String imageRep ="";
        int[] imageRepIndex = messageBody.getImageRepIndex();
        int[] imageRepDispTime = messageBody.getImageRepDispTime();
        for (int i = 0; i < imageRepIndex.length; i++) {
            if(imageRepIndex[i]!=0) {
                imageRep += imageRepIndex[i] + "-" + imageRepDispTime[i] + ",";
            }
        }
        if(imageRep.length()>0) {
            imageRep = imageRep.substring(0, imageRep.length() - 1);
        }
        obj.setImageRep(imageRep);

        // 时间戳 * 1000 => Date
        Date utcDate = new Date(messageBody.getLocalTime() * 1000);
        // utc转北京时间
        obj.setLocalTime(DateUtils.addHours(utcDate, -8));

        // byte => char
        //开始时间
        byte[] openDvTime = messageBody.getOpenDvTime();
        //结束时间
        byte[] stopDvTime = messageBody.getStopDvTime();
        StringBuilder timeStr = new StringBuilder();
        if (openDvTime != null && openDvTime.length > 0) {
            for (int i = 0; i < openDvTime.length; i++) {
                timeStr.append((char) openDvTime[i]);
            }

        }
        if (stopDvTime != null && stopDvTime.length > 0) {
            timeStr.append("-");
            for (int i = 0; i < stopDvTime.length; i++) {
                timeStr.append((char) stopDvTime[i]);
            }
        }
        obj.setOpenDvTime(timeStr.toString());
        return obj;
    }


    /**
     * 下载图片
     */
    public byte[] imageReply(AdHeartBeatPackage beatPack) {
        String imei=beatPack.getImeiStr();
        //图片传输状态回复
        if (beatPack.getDeviceInfo().getImageStatus() != -1) {
            log.info("imei:{}，图片传输状态回复：{}。。。<br/>", imei, beatPack.getDeviceInfo().getImageStatus() == 0 ? "成功" : "失败");
            List<BaseTag> tagList = new LinkedList<>();
            // 硬件版本
            tagList.add(new HardwareVersionTag().setTagValue(new byte[]{0x10}));
            // 不论图片传输成功还是失败，只要接收到，就回复0x00
            tagList.add(new ImageTransmitStatusTag().setTagValue(new byte[]{0x00}));
            return BaseTag.reply(beatPack, McuBeatIdentificationConstant.BEAT_IDENTIFICATION_0X80,
                    McuBeatPackTypeConstant.IMAGE_TRANSMIT_PACKAGE_TYPE, tagList);
        }

        CarouselPlanRedisDto carouselPlanCache = getcarouselPlanCache(imei);

        Long imageIndex = Long.valueOf(beatPack.getDeviceInfo().getImageIndex());
        Set<MaterialLscUrlDto> materials = new LinkedHashSet<>();
        materials.addAll(carouselPlanCache.getAdMaterialsMorning());
        materials.addAll(carouselPlanCache.getAdMaterialsEvening());
        materials.addAll(carouselPlanCache.getAdMaterialsNormal());
        //文件路径
        String imagePathHead = "";
        // 获取LSC文件路径
        for (MaterialLscUrlDto material : materials) {
            if (material.getId().equals(imageIndex)){
                imagePathHead = RuoYiConfig.getProfile() + material.getConvertedFileUrl().replace("profile/", "");
                break;
            }
        }

        if (!new File(imagePathHead).exists()) {
            log.error("请求下载的图片文件不存在！要下载图片:{}，地址:{}，计划返回的图片:{}",imageIndex,imagePathHead,materials);
            return null;
        }
        // 读取图片到字节数组 后面存到redis里面
        FileReader fileReader = new FileReader(imagePathHead);
        byte[] infoImage = fileReader.readBytes();
        // 图片传输

        byte[] imeiBytes = DataTransitionUtil.intsToBytes(beatPack.getIMEI());
        String imeiStr = Convert.toHex(imeiBytes);
        log.info("图片下载请求：索引:{},位置:{},当前包序号为：{}。。。。。<br/>",
                beatPack.getDeviceInfo().getImageIndex(),
                beatPack.getDeviceInfo().getImageSpace(),
                beatPack.getPackageSeq());
        int len = beatPack.getDeviceInfo().getImageLen();
        // info数组下标
        int i = 0;
        byte[] info = new byte[len + 39];
        // 包头
        info[i++] = McuBeatPackHeadAndTailConstant.PACKAGE_HEAD;
        // IMEI
        System.arraycopy(imeiBytes, 0, info, i, imeiBytes.length);
        i += imeiBytes.length;
        // 协议版本
        info[i++] = 0x10;
        // 设备类型
        byte[] equipmentType = DataTransitionUtil.intsToBytes(beatPack.getEquipmentType());
        System.arraycopy(equipmentType, 0, info, i, equipmentType.length);
        i += equipmentType.length;
        // 包序号
        info[i++] = (byte) ((beatPack.getPackageSeq() >> 8) & 0xff);
        info[i++] = (byte) (beatPack.getPackageSeq() & 0xff);
        // 标识
        info[i++] = McuBeatIdentificationConstant.BEAT_IDENTIFICATION_0X00;
        // 包类型
        info[i++] = McuBeatPackTypeConstant.IMAGE_TRANSMIT_PACKAGE_TYPE;
        // 心跳包回复标签数量
        info[i++] = 0x02;
        // 消息体长度
        int msgLen = len + 17;
        info[i++] = (byte) ((msgLen >> 8) & 0xff);
        info[i++] = (byte) ((msgLen) & 0xff);
        int msgLenIndex = i; //消息体长度下标
        // 硬件版本
        info[i++] = McuBeatPackTagConstant.TAG_DEV_HW_REV;
        info[i++] = 0x00;
        info[i++] = 0x01;
        info[i++] = 0x10;
        // 下载图片文件传输
        info[i++] = McuBeatPackTagConstant.TAG_DEV_IMAGE_DOWN_DATA;
        int lenTag21 = len + 10;
        // tag21消息的长度
        info[i++] = (byte) ((lenTag21 >> 8) & 0xff);
        info[i++] = (byte) ((lenTag21) & 0xff);
        // imageIndex
        int imageIndex2 = beatPack.getDeviceInfo().getImageIndex();
        i = getImageIndexAndSpace(i, info, imageIndex2);
        // imageSpace
        int imageSpace = beatPack.getDeviceInfo().getImageSpace();
        i = getImageIndexAndSpace(i, info, imageSpace);
        info[i++] = (byte) ((len >> 8) & 0xff);
        info[i++] = (byte) ((len) & 0xff);
        System.arraycopy(infoImage, imageSpace, info, i, len);
        i += len;

        return getCrc16Data(i, info, beatPack, msgLenIndex);
    }

    /**
     * SE平台业务
     */
    public byte[] seBeatReply(AdHeartBeatPackage beatPack) {
        int len = 0;
        // info数组下标
        int i = 0;
        byte[] info = new byte[len + 3000];
        // 包头
        info[i++] = McuBeatPackHeadAndTailConstant.PACKAGE_HEAD;
        // IMEI
        byte[] imeiBytes = DataTransitionUtil.intsToBytes(beatPack.getIMEI());
        String imeiStr = Convert.toHex(imeiBytes);
        System.arraycopy(imeiBytes, 0, info, i, imeiBytes.length);
        i += imeiBytes.length;
        // 协议版本
        info[i++] = (byte) beatPack.getProtocolVersion();
        // 设备类型
        byte[] equipmentType = DataTransitionUtil.intsToBytes(beatPack.getEquipmentType());
        System.arraycopy(equipmentType, 0, info, i, equipmentType.length);
        i += equipmentType.length;
        // 包序号
        info[i++] = (byte) ((beatPack.getPackageSeq() >> 8) & 0xff);
        info[i++] = (byte) (beatPack.getPackageSeq() & 0xff);
        // 标识
        info[i++] = McuBeatIdentificationConstant.BEAT_IDENTIFICATION_0X00;
        // 包类型
        info[i++] = McuBeatPackTypeConstant.SE_PACKAGE_TYPE;
        // 心跳包回复标签数量
        info[i++] = 0x01;
        // 消息体长度
        int msgLen = len + McuOtherConstant.BEAT_PACKAGE_REPLY_ELSE_LENGTH;
        //消息体长度下标
        int msgLenIndex = i;
        info[i++] = (byte) ((msgLen >> 8) & 0xff);
        info[i++] = (byte) ((msgLen) & 0xff);
        if (beatPack.getDeviceInfo().getSeActivatingTag() == McuBeatPackTagConstant.TAG_DEV_SE_ACTIVATION_UP) {
            String imei = Convert.toHex(DataTransitionUtil.intsToBytes(beatPack.getIMEI()));
            log.info("进入了设备激活，正在请求复旦微安全芯片返回激活数据。。。<br/>");

            info[i++] = McuBeatPackTagConstant.TAG_DEV_SE_ACTIVATION_DOWN;
            byte[] apdu = Convert.hexToBytes(seActivating(beatPack));
            log.info("已收到激活数据，正在封装数据包。。。<br/>");
            log.info("设备：" + imei + "已激活。<br/>");

            // 保存SEID
            if (beatPack.getDeviceInfo().getSeActivating() != null) {
                adHandleService.updateSeId(imei, beatPack.getDeviceInfo().getSeActivating().getSeId());
            }

            info[i++] = (byte) ((apdu.length >> 8) & 0xff);
            info[i++] = (byte) ((apdu.length) & 0xff);
            System.arraycopy(apdu, 0, info, i, apdu.length);
            i += apdu.length;
        } else { // TODO SM4秘钥分配 二期再做
            log.info("进入了SM4秘钥分配，正在请求复旦微安全芯片返回秘钥数据。。。<br/>");
            log.info("已收到秘钥数据，正在封装数据包。。。<br/>");
            log.info("SM4秘钥已储存并返回给设备。<br/>");
            info[i++] = McuBeatPackTagConstant.TAG_DEV_SE_DOWN;
            info[i++] = 0x00;
            info[i++] = 0x17;
            info[i++] = 0x02;
            info[i++] = 0x61;
            info[i++] = 0x08;
            info[i++] = (byte) 0x9F;
            info[i++] = (byte) 0xF9;
            info[i++] = 0x00;
            info[i++] = 0x05;
            info[i++] = 0x01;
            info[i++] = 0x02;
            info[i++] = 0x03;
            info[i++] = 0x04;
            info[i++] = 0x05;
            info[i++] = 0x06;
            info[i++] = 0x07;
            info[i++] = 0x08;
            info[i++] = 0x09;
            info[i++] = 0x0A;
            info[i++] = 0x0B;
            info[i++] = 0x0C;
            info[i++] = 0x0D;
            info[i++] = 0x0E;
            info[i++] = 0x0F;
            info[i++] = 0x10;
        }
        info[msgLenIndex++] = (byte) (((i - 19) >> 8) & 0xff);
        info[msgLenIndex++] = (byte) (((i - 19)) & 0xff);
        return getCrc16Data(i, info, beatPack, msgLenIndex);

    }

    /**
     * 升级包回复
     */
    public byte[] upgradeReply(AdHeartBeatPackage beatPack) throws IOException {
        String imei=beatPack.getImeiStr();
        //固件升级状态
        if (beatPack.getDeviceInfo().getFirewareStatus() != -1) {
            log.info("imei:{}，固件升级状态回复：{}。。。<br/>", imei, beatPack.getDeviceInfo().getFirewareStatus() == 0 ? "成功" : "失败");
            List<BaseTag> tagList = new LinkedList<>();
            // 硬件版本
            tagList.add(new HardwareVersionTag().setTagValue(new byte[]{0x10}));
            // 不论固件升级成功还是失败，只要接收到，就回复0x00
            tagList.add(new FirmwareUpgradeStatusTag().setTagValue(new byte[]{0x00}));
            return  BaseTag.reply(beatPack, McuBeatIdentificationConstant.BEAT_IDENTIFICATION_0X80,
                    McuBeatPackTypeConstant.UPGRADE_PACKAGE_TYPE, tagList);
        }
        // 固件包下载

        log.info("imei:{}，下载升级包数据，当前包序号为：{}。。。<br/>", imei, beatPack.getPackageSeq());
        // 根据屏幕类型、硬件版本查询最新的固件包
        AdMessageBody deviceInfo = beatPack.getDeviceInfo();
        String screenType = Convert.toHex(DataTransitionUtil.intsToBytes(beatPack.getEquipmentType()));
        Integer hardwareVersion = Integer.parseInt(Integer.toHexString(beatPack.getDeviceInfo().getHw())) ;
        AdFirmwareUpgrade param = new AdFirmwareUpgrade().setScreenType(screenType).setHardwareVersion(hardwareVersion).setPublishStatus("1");
        AdFirmwareUpgrade adFirmwareUpgrade = null;
        if (adFirmwareUpgrade == null) {
            return null;
        }
        String filePath = RuoYiConfig.getProfile() + adFirmwareUpgrade.getAttachment().replace("/profile", "");
        FileReader fileReader = new FileReader(filePath);
        byte[] fileDatas = fileReader.readBytes();

        // 封装回复数据
        // 请求下载的固件包长度
        int reqDownFileLength = deviceInfo.getFirmwareLength();
        // 固件包数据 + 35个长度的字节数据
        byte[] resDatas = new byte[reqDownFileLength + 35];
        int index = 0;
        // 包头
        resDatas[index++] = McuBeatPackHeadAndTailConstant.PACKAGE_HEAD;
        // 唯一编码 IMEI
        byte[] imeiDatas = DataTransitionUtil.intsToBytes(beatPack.getIMEI());
        System.arraycopy(imeiDatas, 0, resDatas, index, imeiDatas.length);
        index += imeiDatas.length;
        // 协议版本
        resDatas[index++] = 0x10;
        // 设备类型
        byte[] equipmentTypeDatas = DataTransitionUtil.intsToBytes(beatPack.getEquipmentType());
        System.arraycopy(equipmentTypeDatas, 0, resDatas, index, equipmentTypeDatas.length);
        index += equipmentTypeDatas.length;
        // 包序号
        resDatas[index++] = (byte) ((beatPack.getPackageSeq() >> 8) & 0xff);
        resDatas[index++] = (byte) (beatPack.getPackageSeq() & 0xff);
        // 标识
        resDatas[index++] = McuBeatIdentificationConstant.BEAT_IDENTIFICATION_0X00;
        // 包类型
        resDatas[index++] = McuBeatPackTypeConstant.UPGRADE_PACKAGE_TYPE;
        // 标签数量 2个标签
        resDatas[index++] = 0x02;
        // 消息体总长度 固件包数据长度 + 13个字节的长度
        int msgBodyLength = reqDownFileLength + 13;
        resDatas[index++] = (byte) ((msgBodyLength >> 8) & 0xff);
        resDatas[index++] = (byte) ((msgBodyLength) & 0xff);
        // 消息体数据开始的下标位置，在crc16校验时使用
        int msgBodyStartIndex = index;
        // 以下是消息体的内容
        // 硬件版本
        resDatas[index++] = McuBeatPackTagConstant.TAG_DEV_HW_REV;
        resDatas[index++] = 0x00;
        resDatas[index++] = 0x01;
        resDatas[index++] = Convert.hexToBytes(adFirmwareUpgrade.getHardwareVersion().toString())[0];
        // tlv t:下载升级文件传输
        resDatas[index++] = McuBeatPackTagConstant.TAG_DEV_UPGRADE_TRANSFER;
        // tlv l:t（0x18）的长度 固件包数据长度 + 6个字节的长度
        int tagUpgradeTransferLength = reqDownFileLength + 6;
        resDatas[index++] = (byte) ((tagUpgradeTransferLength >> 8) & 0xff);
        resDatas[index++] = (byte) ((tagUpgradeTransferLength) & 0xff);
        // tlv v:固件包起始位置+数据长度+数据
        int firmwareIndex = deviceInfo.getFirmwareIndex();
        // 固件包起始位置占4个字节
        resDatas[index++] = (byte) ((firmwareIndex >> 24) & 0xff);
        resDatas[index++] = (byte) ((firmwareIndex >> 16) & 0xff);
        resDatas[index++] = (byte) ((firmwareIndex >> 8) & 0xff);
        resDatas[index++] = (byte) ((firmwareIndex) & 0xff);
        // 数据长度2个字节
        resDatas[index++] = (byte) ((reqDownFileLength >> 8) & 0xff);
        resDatas[index++] = (byte) ((reqDownFileLength) & 0xff);
        // 拷贝固件包对应位置的数据
        System.arraycopy(fileDatas, firmwareIndex, resDatas, index, reqDownFileLength);
        index += reqDownFileLength;
        // 回复
        return getCrc16Data(index, resDatas, beatPack, msgBodyStartIndex);
    }

    /**
     * 获取激活数据
     *
     * @param beatPack
     * @return
     */
    private String seActivating(AdHeartBeatPackage beatPack) {
        //应用方流水号
        String apTxnNo = IdUtil.simpleUUID();
        String imei = Convert.toHex(DataTransitionUtil.intsToBytes(beatPack.getIMEI()));

        SeActivating seActivating = beatPack.getDeviceInfo().getSeActivating();
        seActivating.setApTxnNo(apTxnNo).setDeviceId(imei).setDeviceInfo(imei);

        log.info("seActivating: " + seActivating);
        return seActivatingService.activating(apTxnNo, seActivating);
    }

    /**
     * 获取图片列表
     *
     * @param imageSeq
     * @param info
     * @param i
     * @param imageCn
     * @return
     */
    private int getImageList(Integer[] imageSeq, byte[] info, int i, int imageCn) {
        info[i++] = (byte) ((imageSeq[imageCn] >> 24) & 0xff);
        info[i++] = (byte) ((imageSeq[imageCn] >> 16) & 0xff);
        info[i++] = (byte) ((imageSeq[imageCn] >> 8) & 0xff);
        info[i++] = (byte) ((imageSeq[imageCn]) & 0xff);
        return i;
    }

    /**
     * 获取轮播时间间隔
     *
     * @param info
     * @param i
     * @param carouselTime
     * @return
     */
    private int getCarouselTime(byte[] info, int i, char[] carouselTime) {
        info[i++] = (byte) ((((carouselTime[0] - 0x30) << 4) & 0xf0) + (((carouselTime[1] - 0x30)) & 0x0f));
        info[i++] = (byte) ((((carouselTime[2] - 0x30) << 4) & 0xf0) + (((carouselTime[3] - 0x30)) & 0x0f));   //
        info[i++] = (byte) ((((carouselTime[4] - 0x30) << 4) & 0xf0) + (((carouselTime[5] - 0x30)) & 0x0f));        //
        return i;
    }

    /**
     * 获取加入crc16以及包尾的data
     *
     * @param i
     * @param info
     * @return
     */
    private byte[] getCrc16Data(int i, byte[] info, AdHeartBeatPackage beatPack, int msgLenIndex) {
        byte[] temp = new byte[i];
        System.arraycopy(info, 0, temp, 0, temp.length);
        long crc16Value = CRC16ModbusUtil.getValue(temp);
        info[i++] = (byte) ((crc16Value >> 8) & 0xff);
        info[i++] = (byte) ((crc16Value) & 0xff);
        info[i++] = McuBeatPackHeadAndTailConstant.PACKAGE_TAIL;
        if ((beatPack.getIdentification() & 0x01) == 0x01) {
            int lenBuf = ((info[msgLenIndex - 2] < 0 ? info[msgLenIndex - 2] + 256 : info[msgLenIndex - 2]) << 8)
                    +
                    (info[msgLenIndex - 1] < 0 ? info[msgLenIndex - 1] + 256 : info[msgLenIndex - 1]);
            byte[] bBuffer3 = new byte[lenBuf];
            System.arraycopy(info, msgLenIndex, bBuffer3, 0, lenBuf);
            // sm4加密
            SM4Util.payloadEncrypt(bBuffer3, Convert.hexToBytes(McuSm4KeyConfig.SM4KEY), DataTransitionUtil.intsToBytes(beatPack.getIMEI()), beatPack.getPackageSeq());
            System.arraycopy(bBuffer3, 0, info, msgLenIndex, lenBuf);
        }
        byte[] data = new byte[i];
        System.arraycopy(info, 0, data, 0, i);
        // 调试时，注释sm4加密代码，放开以下代码
//        log.info(Convert.toHex(data));
        return data;
    }

    private int getImageIndexAndSpace(int i, byte[] info, int data) {
        info[i++] = (byte) ((data >> 24) & 0xff);
        info[i++] = (byte) ((data >> 16) & 0xff);
        info[i++] = (byte) ((data >> 8) & 0xff);
        info[i++] = (byte) ((data) & 0xff);
        return i;
    }

    /**
     * 计算拉手和心跳是否异常
     * @param num
     * @return
     */
    private int calculateStatus(Integer num){
        int status = 0;
        if (num!=null&&num != 0&&num!=8) {
            status=-1;
        }
        return status;
    }

    /**
     * 获取redis中的缓存计划
     * @param imei
     * @return
     */
    public CarouselPlanRedisDto getcarouselPlanCache(String imei){
        // 获取轮播数据
        CarouselPlanRedisDto carouselPlanCache= adCarouselPlanService.getCarouselPlanCache(SubwayConstant.SUBWAY_ID.toString());
        if (carouselPlanCache == null) {
            log.error("获取轮播计划缓存发生错误：未查询到轮播计划信息。");
        }
        return carouselPlanCache;
    }

}
