package com.ruoyi.mcu.util;

import cn.hutool.core.io.checksum.crc16.CRC16Checksum;
import cn.hutool.core.io.checksum.crc16.CRC16Modbus;
import org.springframework.stereotype.Component;


@Component
/**
 * 数据校验工具类
 * @author zzw
 * @date 2021/6/7
 * @Description: crc16数据校验
 */
public class CRC16ModbusUtil {
    private static CRC16Checksum checksum = new CRC16Modbus();

    /**
     * 获取CRC16Modbus校验码
     * @param data 待生成校验码的数据
     * @param isPadding 是否补0
     * @return CRC16Modbus校验码
     */
    public static String getHexValue(byte[] data, boolean isPadding){
        checksum.reset();
        checksum.update(data);
        return checksum.getHexValue(isPadding);
    }

    /**
     * 获取CRC16Modbus校验码的Long类型数据
     * @param data 待生成校验码的数据
     * @return CRC16Modbus校验码
     */
    public static long getValue(byte[] data){
        checksum.reset();
        checksum.update(data);
        return checksum.getValue();
    }

    /**
     * 校验crc16
     * @param packetData 待校验的数据
     * @param crc16 校验码
     * @return 校验结果
     */
    public static boolean getCheckResult(int[] packetData,Long crc16){
        // 转为byte数组
        byte[] data = DataTransitionUtil.intsToBytes(packetData);
        // 得到待校验的数据
        checksum.reset();
        checksum.update(data);
        return crc16 == checksum.getValue();
    }
}
