package com.ruoyi.mcu.util;

import cn.hutool.crypto.symmetric.SymmetricCrypto;

/**
 * SM4加密解密工具类
 *
 * @author zzw
 * @date 2021年8月31日 10:36:32
 * @Description: 国密算法
 */
public class SM4Util {

    /**
     * 消息体加解密过程，通过调用此函数可将待加密数据进行加密，需要解密的数据进行解密的逆向过程
     * 加密过程：通过对一个16字节的数据进行SM4加密后产生 一个16字节的加密因子，然后用这个16字节的加密因子同待处理数据进行异或操作从而产生新的数据到达加密的作用，解密则相反从而取出解密数据
     *
     * @param data       待处理数据
     * @param key        密钥，长度为16字节
     * @param imei       设备的IMEI，可以从消息的imei中获取
     * @param packageSeq 消息的帧计数，可以从消息中获取，发送时使用发送消息中的packageSeq
     * @return 加密后的数据
     */
    public static byte[] payloadEncrypt(byte[] data, byte[] key, byte[] imei, int packageSeq) {
        int size = data.length; // 待处理数据长度
        int dataIndex = 0; // 数据数组下标
        int ctr = 1;
        int i;
        byte[] aBlock = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
        byte[] sBlock = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

        // 加密，这里调用BouncyCastle库进行SM4运算产生对象。
        SymmetricCrypto sm4 = new SymmetricCrypto("SM4/ECB/NoPadding", key);
        //将设备的IMEI和packageSeq 放到待SM4加密的数据中
        aBlock[0] = 0x01;
        for (i = 0; i < 8; i++) {
            aBlock[1 + i] = imei[i];
        }
        aBlock[9] = (byte) ((packageSeq >> 8) & 0xff);
        aBlock[10] = (byte) ((packageSeq) & 0xff);

        //调用加密接口将aBlock进行加密，产生一个16字节的sBlock 加密因子数据
        sBlock = sm4.encrypt(aBlock);

        // 16的整数倍数据处理过程
        while (size >= 16) {
            aBlock[15] = (byte) ((ctr) & 0xff);
            ctr++;
            sBlock = sm4.encrypt(aBlock);// 重新生成加密因子
            for (i = 0; i < 16; i++) {
                // 同待处理数据进行异或操作从而产生新的数据到达加密的效果
                data[dataIndex + i] = (byte) ((data[dataIndex + i] < 0 ? data[dataIndex + i]+256 : data[dataIndex + i]) ^ (sBlock[i]<0?sBlock[i]+256 : sBlock[i]));
            }
            size -= 16;
            dataIndex += 16;
        }

        //16整除后余下的数据处理
        if (size > 0) {
            aBlock[15] = (byte) ((ctr) & 0xff);
            sBlock = sm4.encrypt(aBlock);
            for (i = 0; i < size; i++) {
                data[dataIndex + i] = (byte) ((data[dataIndex + i] < 0 ? data[dataIndex + i]+256 : data[dataIndex + i]) ^ (sBlock[i]<0?sBlock[i]+256 : sBlock[i]));
            }
        }
        return data;
    }
}
