package com.ruoyi.setsm.config;

import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.LayeredConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.ssl.SSLContexts;
import org.apache.http.ssl.TrustStrategy;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import javax.net.ssl.SSLContext;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.cert.CertificateFactory;

/**
 * @author: jd
 * @Description:
 * @create: 2022/12/8
 */
@Component
@Configuration
public class RestTemplateConfig {

    @Resource
    private SE_TSMKeyConfig se_tsmKeyConfig;

    @Bean
    public RestTemplate restTemplate(SimpleClientHttpRequestFactory simpleClientHttpRequestFactory){
        return new RestTemplate(simpleClientHttpRequestFactory);
    }

    @Bean
    public SimpleClientHttpRequestFactory simpleClientHttpRequestFactory(){
        SimpleClientHttpRequestFactory factory = new SimpleClientHttpRequestFactory();
        factory.setConnectTimeout(15000);
        factory.setReadTimeout(5000);
        return factory;
    }

    @Bean(name = "restTemplateForAuthentication")
    public RestTemplate restTemplateForAuthentication() throws Exception {
        RestTemplate restTemplate;
        if ("dev".equals(se_tsmKeyConfig.getEnvironment())){
            restTemplate = new RestTemplate(this.singleAuthenticationClientHttpRequestFactory());
        }else {
            restTemplate = new RestTemplate(this.mutualAuthenticationClientHttpRequestFactory());
        }
        return restTemplate;
    }

    public HttpComponentsClientHttpRequestFactory singleAuthenticationClientHttpRequestFactory() throws Exception{
        TrustStrategy acceptingTrustStrategy = (x509Certificates, authType) -> true;
        SSLContext sslContext = SSLContexts.custom().loadTrustMaterial(null, acceptingTrustStrategy).build();
        LayeredConnectionSocketFactory csf = new SSLConnectionSocketFactory(sslContext, new NoopHostnameVerifier());

        Registry<ConnectionSocketFactory> sfr = RegistryBuilder.<ConnectionSocketFactory>create().register("http", PlainConnectionSocketFactory.getSocketFactory()).register("https", csf != null ? csf : SSLConnectionSocketFactory.getSocketFactory()).build();
        PoolingHttpClientConnectionManager pollingConnectionManager = new PoolingHttpClientConnectionManager(sfr);

        CloseableHttpClient httpClient = HttpClients.custom().setSSLSocketFactory(csf).setConnectionManager(pollingConnectionManager).build();
        HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory();
        factory.setHttpClient(httpClient);
        factory.setConnectTimeout(5000);
        factory.setReadTimeout(6000);
        factory.setConnectionRequestTimeout(60000);
        return factory;
    }

    public HttpComponentsClientHttpRequestFactory mutualAuthenticationClientHttpRequestFactory() throws Exception{
        TrustStrategy acceptingTrustStrategy = (x509Certificates, authType) -> true;

        // 建立TLS连接
        SSLContext sslContext = null;

        char[] password = se_tsmKeyConfig.getCert().getLocalCertPassword().toCharArray();
        KeyStore keyStore = KeyStore.getInstance("jks");
        KeyStore keyStore2 = KeyStore.getInstance("PKCS12");

        File jks = new File(se_tsmKeyConfig.getCert().getLocalCert());
        File serverCaFile = new File(se_tsmKeyConfig.getCert().getFuDanWeiCaCert());
        InputStream in = new FileInputStream(jks);
        InputStream in2 = new FileInputStream(serverCaFile);

        CertificateFactory cerFactory = CertificateFactory.getInstance("X.509");
        java.security.cert.Certificate cer = cerFactory.generateCertificate(in2);
        keyStore.load(in, password);
        keyStore2.load(null,null);
        keyStore2.setCertificateEntry("fudanwei", cer);

        sslContext = SSLContexts.custom()
                // 配置信任链，用于跳过验证服务器证书
                .loadTrustMaterial(keyStore2, acceptingTrustStrategy)
                .loadKeyMaterial(keyStore, password)
                .build();

        SSLConnectionSocketFactory connectionSocketFactory = new SSLConnectionSocketFactory(sslContext, new NoopHostnameVerifier());

        HttpClientBuilder httpClientBuilder = HttpClients.custom();
        httpClientBuilder.setSSLSocketFactory(connectionSocketFactory);
        CloseableHttpClient httpClient = httpClientBuilder.build();
        HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory();
        factory.setHttpClient(httpClient);
        return factory;
    }
}
