package com.ruoyi.setsm.config;

import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

/**
 * @author: jd
 * @Description:
 * @create: 2022/12/7
 */
@Data
@Component
@ConfigurationProperties(prefix = "se")
public class SE_TSMKeyConfig {

    /**
     * 复旦微通信 RSA秘钥
     */
    private Key key;

    /**
     * 复旦微SE-TESM
     */
    private SeTesm seTesm;

    /**
     * 证书地址
     */
    private Cert cert;

    /**
     * 认证方式：dev单向认证，prod双向认证
     */
    private String environment;

    @Data
    @NoArgsConstructor
    public static class Key{
        /**
         * se-tsm公钥
         */
        private String sePubKey;

        /**
         * 平台私钥
         */
        private String priKey;

        /**
         * 平台公钥
         */
        private String pubKey;
    }

    @Data
    @NoArgsConstructor
    public static class SeTesm{
        /**
         * 复旦微SE-TESM 协议
         */
        private String protocol;

        /**
         * 复旦微SE-TESM 主机和端口
         */
        private String hostAndPort;

        /**
         * DMS系统通过该接口向SE-TSM系统请求激活设备的URL
         */
        private String activatingUrl;

        /**
         * DMS系统通过该接口向SE-TSM系统请求签名的URL
         */
        private String signatureUrl;
    }

    @Data
    @NoArgsConstructor
    public static class Cert{
        /**
         * 复旦微中间证书
         */
        private String fuDanWeiCaCert;

        /**
         * 我们本地的证书
         */
        private String localCert;

        /**
         * 我们本地证书的密码
         */
        private String localCertPassword;
    }
}
