package com.ruoyi.setsm.service.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.advertiser.constant.SeSignature;
import com.ruoyi.setsm.config.SE_TSMKeyConfig;
import com.ruoyi.setsm.domain.ActivatingCommand;
import com.ruoyi.setsm.domain.SeActivating;
import com.ruoyi.setsm.domain.SeResult;
import com.ruoyi.setsm.domain.SeSignatureResult;
import com.ruoyi.setsm.exception.McuServiceException;
import com.ruoyi.setsm.service.SeActivatingService;
import com.ruoyi.setsm.util.SignatureUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

/**
 * @author: jd
 * @Description:
 * @create: 2022/12/7
 */
@Slf4j
@Service
public class SeActivatingServiceImpl implements SeActivatingService {

    @Resource
    private SE_TSMKeyConfig se_tsmKeyConfig;

    @Resource(name = "restTemplateForAuthentication")
    private RestTemplate restTemplateForAuthentication;

    /**
     * 激活业务接口
     *
     * @param requestId    请求标识，作为判断重发报文的唯一标志
     * @param seActivating Activating – 激活业务码数据
     */
    @Override
    public String activating(String requestId, SeActivating seActivating) {

        //se-tsm公钥
        String sePubKey = se_tsmKeyConfig.getKey().getSePubKey();
        String pubKey = se_tsmKeyConfig.getKey().getPubKey();
        //平台私钥
        String priKey = se_tsmKeyConfig.getKey().getPriKey();

        StringBuffer url = new StringBuffer();
        url.append(se_tsmKeyConfig.getSeTesm().getProtocol());
        url.append(se_tsmKeyConfig.getSeTesm().getHostAndPort());
        url.append(se_tsmKeyConfig.getSeTesm().getActivatingUrl());
        JSONObject activating = JSONObject.parseObject(JSONObject.toJSONString(seActivating));
        log.debug("activating", activating);
        String bizData = Base64.encode(activating.toString());

        //签名数据
        byte[] signature = SignatureUtil.sign(bizData.getBytes(), priKey);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.add("request-id", requestId);
        JSONObject data = new JSONObject();
        data.put("bizData", bizData);
        data.put("signature", signature);
        JSONObject bizSignedData = new JSONObject();
        bizSignedData.put("bizSignedData", data);

        log.debug("bizSignedData", bizSignedData);
        HttpEntity<JSONObject> httpEntity = new HttpEntity<JSONObject>(bizSignedData, headers);

        Object activatingCommands = null;//restTemplateUtil.sendPost(url.toString(), httpEntity);

        log.debug("activatingCommands", activatingCommands);

        SeResult seResult = JSONUtil.toBean(activatingCommands.toString(), SeResult.class);
        log.debug("seResult", seResult);
        if(null==seResult.getBizSignedData()){
            log.error("seResult:",seResult);
            throw new McuServiceException("激活失败，接收到的bizSignedData为空");
        }

        StringBuffer activatingData = new StringBuffer();//se激活数据
        //验签
        boolean verify = Optional.ofNullable(SignatureUtil.verify(seResult.getBizSignedData().getBizData().getBytes(), Base64.decode(seResult.getBizSignedData().getSignature()), sePubKey)).orElse(false);
        if (verify) {
            String decodeStr = Base64.decodeStr(seResult.getBizSignedData().getBizData());
            log.debug(decodeStr);
            JSON parse = JSONUtil.parse(decodeStr);
            Object activatingCommands1 = parse.getByPath("activatingCommands");
            String aa = JSONObject.toJSONString(activatingCommands1);
            List<ActivatingCommand> activatingCommands2 = null;//JSONArray.parseArray(aa, ActivatingCommand.class);

            activatingCommands2.forEach(activatingCommand -> {
                log.debug("activatingCommand", activatingCommand);
                activatingData.append(activatingCommand.getApdu());
            });
        } else {
            throw new McuServiceException("验签失败!");
        }

        return activatingData.toString();
    }

    @Override
    public String signature(String requestId, SeSignature seSignature) {
        // se-tsm公钥
        String sePubKey = se_tsmKeyConfig.getKey().getSePubKey();
        // 平台私钥
        String priKey = se_tsmKeyConfig.getKey().getPriKey();

        // 签名数据
        JSONObject activating = JSONObject.parseObject(JSONObject.toJSONString(seSignature));
        String bizData = Base64.encode(activating.toString());
        byte[] signature = SignatureUtil.sign(bizData.getBytes(), priKey);

        // 请求参数
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.add("request-id", requestId);
        JSONObject data = new JSONObject();
        data.put("bizData", bizData);
        data.put("signature", signature);
        JSONObject bizSignedData = new JSONObject();
        bizSignedData.put("bizSignedData", data);
        StringBuffer url = new StringBuffer();
        url.append(se_tsmKeyConfig.getSeTesm().getProtocol());
        url.append(se_tsmKeyConfig.getSeTesm().getHostAndPort());
        url.append(se_tsmKeyConfig.getSeTesm().getSignatureUrl());
        HttpEntity<JSONObject> httpEntity = new HttpEntity<JSONObject>(bizSignedData, headers);
        SeResult seResult = restTemplateForAuthentication.postForObject(url.toString(), httpEntity, SeResult.class);
        log.debug("activatingCommands", seResult);

        //验签
        boolean verify = SignatureUtil.verify(seResult.getBizSignedData().getBizData().getBytes(), Base64.decode(seResult.getBizSignedData().getSignature()), sePubKey);
        if (verify) {
            String decodeStr = Base64.decodeStr(seResult.getBizSignedData().getBizData());
            SeSignatureResult seSignatureResult = JSONUtil.toBean(decodeStr,SeSignatureResult.class);
            log.debug("seSignatureResult", seSignatureResult);
            return seSignatureResult.getSignResult();
        } else {
            throw new McuServiceException("验签失败!");
        }
    }
}
